# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.allowed_output_formats import AllowedOutputFormats
from ..types.music_upload_response import MusicUploadResponse
from .raw_client import AsyncRawMusicClient, RawMusicClient
from .types.body_compose_music_v_1_music_post_composition_plan import BodyComposeMusicV1MusicPostCompositionPlan
from .types.body_compose_music_v_1_music_post_model_id import BodyComposeMusicV1MusicPostModelId
from .types.body_compose_music_with_a_detailed_response_v_1_music_detailed_post_composition_plan import (
    BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan,
)
from .types.body_compose_music_with_a_detailed_response_v_1_music_detailed_post_model_id import (
    BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId,
)
from .types.body_stream_composed_music_v_1_music_stream_post_composition_plan import (
    BodyStreamComposedMusicV1MusicStreamPostCompositionPlan,
)
from .types.body_stream_composed_music_v_1_music_stream_post_model_id import (
    BodyStreamComposedMusicV1MusicStreamPostModelId,
)
from .types.body_stream_composed_music_with_a_detailed_response_v_1_music_detailed_stream_post_composition_plan import (
    BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostCompositionPlan,
)
from .types.body_stream_composed_music_with_a_detailed_response_v_1_music_detailed_stream_post_model_id import (
    BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostModelId,
)
from .types.music_compose_detailed_request_output_format import MusicComposeDetailedRequestOutputFormat
from .types.music_compose_detailed_stream_request_output_format import MusicComposeDetailedStreamRequestOutputFormat
from .types.music_compose_request_output_format import MusicComposeRequestOutputFormat
from .types.music_separate_stems_request_stem_variation_id import MusicSeparateStemsRequestStemVariationId
from .types.music_stream_request_output_format import MusicStreamRequestOutputFormat
from .types.music_video_to_music_request_model_id import MusicVideoToMusicRequestModelId

if typing.TYPE_CHECKING:
    from .composition_plan.client import AsyncCompositionPlanClient, CompositionPlanClient
    from .finetunes.client import AsyncFinetunesClient, FinetunesClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MusicClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMusicClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._composition_plan: typing.Optional[CompositionPlanClient] = None
        self._finetunes: typing.Optional[FinetunesClient] = None

    @property
    def with_raw_response(self) -> RawMusicClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMusicClient
        """
        return self._raw_client

    def video_to_music(
        self,
        *,
        videos: typing.List[core.File],
        output_format: typing.Optional[AllowedOutputFormats] = None,
        description: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.List[str]] = OMIT,
        model_id: typing.Optional[MusicVideoToMusicRequestModelId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Generate background music from one or more video files. Videos are combined in order. Optional description and style tags influence the generated music.

        Parameters
        ----------
        videos : typing.List[core.File]
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        description : typing.Optional[str]
            Optional text description of the music you want. A maximum of 1000 characters is allowed.

        tags : typing.Optional[typing.List[str]]
            Optional list of style tags (e.g. ['upbeat', 'cinematic']). A maximum of 10 tags is allowed.

        model_id : typing.Optional[MusicVideoToMusicRequestModelId]
            The model to use for the generation.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Generated audio file matching the video. Content-Type and file extension depend on the output_format parameter (default mp3).
        """
        with self._raw_client.video_to_music(
            videos=videos,
            output_format=output_format,
            description=description,
            tags=tags,
            model_id=model_id,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            yield from r.data

    def compose(
        self,
        *,
        output_format: typing.Optional[MusicComposeRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicV1MusicPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicV1MusicPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            The generated audio file in the format specified

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.music.compose()
        """
        with self._raw_client.compose(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            respect_sections_durations=respect_sections_durations,
            store_for_inpainting=store_for_inpainting,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            yield from r.data

    def compose_detailed(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Multipart/mixed response with JSON metadata and binary audio file

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.music.compose_detailed()
        """
        with self._raw_client.compose_detailed(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            respect_sections_durations=respect_sections_durations,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            yield from r.data

    def compose_detailed_stream(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[str]:
        """
        Stream a song and its detailed metadata using Server-Sent Events (SSE).

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[str]
            Server-Sent Events for composition plan, song metadata, audio chunks with optional word timestamps, and completion.

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        response = client.music.compose_detailed_stream(
            output_format="auto",
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.compose_detailed_stream(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            request_options=request_options,
        ) as r:
            yield from r.data

    def stream(
        self,
        *,
        output_format: typing.Optional[MusicStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Stream a composed song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Streaming audio data in the format specified

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.music.stream()
        """
        with self._raw_client.stream(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            store_for_inpainting=store_for_inpainting,
            request_options=request_options,
        ) as r:
            yield from r.data

    def upload(
        self,
        *,
        file: core.File,
        extract_composition_plan: typing.Optional[str] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MusicUploadResponse:
        """
        Upload a music file to be later used for inpainting. Price for uploading is the same as the one for song generation. All uploaded content gets inspected for copyright infringement. If copyrighted content is detected, half of the request cost is still charged.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        extract_composition_plan : typing.Optional[str]
            Whether to generate and return the composition plan for the uploaded song. Pass a model id (`music_v1` or `music_v2`) to control which composition plan format is returned. Passing `true`/`false` is deprecated; `true` defaults to the `music_v1` plan format. Enabling this will increase the latency.

        with_timestamps : typing.Optional[bool]
            Whether to transcribe the uploaded song and return word-level timestamps. If True, the response will include words_timestamps but will increase the latency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MusicUploadResponse
            Successfully uploaded music file with optional composition plan

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.music.upload()
        """
        _response = self._raw_client.upload(
            file=file,
            extract_composition_plan=extract_composition_plan,
            with_timestamps=with_timestamps,
            request_options=request_options,
        )
        return _response.data

    def separate_stems(
        self,
        *,
        file: core.File,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        stem_variation_id: typing.Optional[MusicSeparateStemsRequestStemVariationId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[bytes]:
        """
        Separate an audio file into individual stems. This endpoint might have high latency, depending on the length of the audio file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        stem_variation_id : typing.Optional[MusicSeparateStemsRequestStemVariationId]
            The id of the stem variation to use.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            ZIP archive containing separated audio stems. Each stem is provided as a separate audio file in the requested output format.
        """
        with self._raw_client.separate_stems(
            file=file,
            output_format=output_format,
            stem_variation_id=stem_variation_id,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            yield from r.data

    @property
    def composition_plan(self):
        if self._composition_plan is None:
            from .composition_plan.client import CompositionPlanClient  # noqa: E402

            self._composition_plan = CompositionPlanClient(client_wrapper=self._client_wrapper)
        return self._composition_plan

    @property
    def finetunes(self):
        if self._finetunes is None:
            from .finetunes.client import FinetunesClient  # noqa: E402

            self._finetunes = FinetunesClient(client_wrapper=self._client_wrapper)
        return self._finetunes


class AsyncMusicClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMusicClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._composition_plan: typing.Optional[AsyncCompositionPlanClient] = None
        self._finetunes: typing.Optional[AsyncFinetunesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawMusicClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMusicClient
        """
        return self._raw_client

    async def video_to_music(
        self,
        *,
        videos: typing.List[core.File],
        output_format: typing.Optional[AllowedOutputFormats] = None,
        description: typing.Optional[str] = OMIT,
        tags: typing.Optional[typing.List[str]] = OMIT,
        model_id: typing.Optional[MusicVideoToMusicRequestModelId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Generate background music from one or more video files. Videos are combined in order. Optional description and style tags influence the generated music.

        Parameters
        ----------
        videos : typing.List[core.File]
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        description : typing.Optional[str]
            Optional text description of the music you want. A maximum of 1000 characters is allowed.

        tags : typing.Optional[typing.List[str]]
            Optional list of style tags (e.g. ['upbeat', 'cinematic']). A maximum of 10 tags is allowed.

        model_id : typing.Optional[MusicVideoToMusicRequestModelId]
            The model to use for the generation.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Generated audio file matching the video. Content-Type and file extension depend on the output_format parameter (default mp3).
        """
        async with self._raw_client.video_to_music(
            videos=videos,
            output_format=output_format,
            description=description,
            tags=tags,
            model_id=model_id,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def compose(
        self,
        *,
        output_format: typing.Optional[MusicComposeRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicV1MusicPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicV1MusicPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicV1MusicPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            The generated audio file in the format specified

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.music.compose()


        asyncio.run(main())
        """
        async with self._raw_client.compose(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            respect_sections_durations=respect_sections_durations,
            store_for_inpainting=store_for_inpainting,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def compose_detailed(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        respect_sections_durations: typing.Optional[bool] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Compose a song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyComposeMusicWithADetailedResponseV1MusicDetailedPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        respect_sections_durations : typing.Optional[bool]
            Controls how strictly section durations in the `composition_plan` are enforced. Only used with `composition_plan` and only applies to `music_v1`; for `music_v2` section durations are always enforced and this is ignored. When false for `music_v1`, the model may adjust individual section durations for better quality and latency, while preserving the total song duration from the plan.

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Multipart/mixed response with JSON metadata and binary audio file

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.music.compose_detailed()


        asyncio.run(main())
        """
        async with self._raw_client.compose_detailed(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            respect_sections_durations=respect_sections_durations,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def compose_detailed_stream(
        self,
        *,
        output_format: typing.Optional[MusicComposeDetailedStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[
            BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostCompositionPlan
        ] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[str]:
        """
        Stream a song and its detailed metadata using Server-Sent Events (SSE).

        Parameters
        ----------
        output_format : typing.Optional[MusicComposeDetailedStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicWithADetailedResponseV1MusicDetailedStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        with_timestamps : typing.Optional[bool]
            Whether to return the timestamps of the words in the generated song.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[str]
            Server-Sent Events for composition plan, song metadata, audio chunks with optional word timestamps, and completion.

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.music.compose_detailed_stream(
                output_format="auto",
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.compose_detailed_stream(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            store_for_inpainting=store_for_inpainting,
            with_timestamps=with_timestamps,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def stream(
        self,
        *,
        output_format: typing.Optional[MusicStreamRequestOutputFormat] = None,
        prompt: typing.Optional[str] = OMIT,
        composition_plan: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan] = OMIT,
        music_length_ms: typing.Optional[int] = OMIT,
        model_id: typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId] = OMIT,
        seed: typing.Optional[int] = OMIT,
        force_instrumental: typing.Optional[bool] = OMIT,
        finetune_id: typing.Optional[str] = OMIT,
        store_for_inpainting: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Stream a composed song from a prompt or a composition plan.

        Parameters
        ----------
        output_format : typing.Optional[MusicStreamRequestOutputFormat]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. Use "auto" (the default) to let the API pick the best format for the selected model: mp3_44100_128 for v1 models and mp3_48000_192 for v2 models.

        prompt : typing.Optional[str]
            A simple text prompt to generate a song from. Cannot be used in conjunction with `composition_plan`.

        composition_plan : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostCompositionPlan]
            A detailed composition plan to guide music generation. Cannot be used in conjunction with `prompt`.

        music_length_ms : typing.Optional[int]
            The length of the song to generate in milliseconds. Used only in conjunction with `prompt`. Must be between 3000ms and 600000ms. Optional - if not provided, the model will choose a length based on the prompt.

        model_id : typing.Optional[BodyStreamComposedMusicV1MusicStreamPostModelId]
            The model to use for the generation.

        seed : typing.Optional[int]
            Random seed to initialize the music generation process. Providing the same seed with the same parameters can help achieve more consistent results, but exact reproducibility is not guaranteed and outputs may change across system updates. Cannot be used in conjunction with prompt.

        force_instrumental : typing.Optional[bool]
            If true, guarantees that the generated song will be instrumental. If false, the song may or may not be instrumental depending on the `prompt`. Can only be used with `prompt`.

        finetune_id : typing.Optional[str]
            The ID of the finetune to use for the generation

        store_for_inpainting : typing.Optional[bool]
            Whether to store the generated song for inpainting.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Streaming audio data in the format specified

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.music.stream()


        asyncio.run(main())
        """
        async with self._raw_client.stream(
            output_format=output_format,
            prompt=prompt,
            composition_plan=composition_plan,
            music_length_ms=music_length_ms,
            model_id=model_id,
            seed=seed,
            force_instrumental=force_instrumental,
            finetune_id=finetune_id,
            store_for_inpainting=store_for_inpainting,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def upload(
        self,
        *,
        file: core.File,
        extract_composition_plan: typing.Optional[str] = OMIT,
        with_timestamps: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MusicUploadResponse:
        """
        Upload a music file to be later used for inpainting. Price for uploading is the same as the one for song generation. All uploaded content gets inspected for copyright infringement. If copyrighted content is detected, half of the request cost is still charged.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        extract_composition_plan : typing.Optional[str]
            Whether to generate and return the composition plan for the uploaded song. Pass a model id (`music_v1` or `music_v2`) to control which composition plan format is returned. Passing `true`/`false` is deprecated; `true` defaults to the `music_v1` plan format. Enabling this will increase the latency.

        with_timestamps : typing.Optional[bool]
            Whether to transcribe the uploaded song and return word-level timestamps. If True, the response will include words_timestamps but will increase the latency.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MusicUploadResponse
            Successfully uploaded music file with optional composition plan

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.music.upload()


        asyncio.run(main())
        """
        _response = await self._raw_client.upload(
            file=file,
            extract_composition_plan=extract_composition_plan,
            with_timestamps=with_timestamps,
            request_options=request_options,
        )
        return _response.data

    async def separate_stems(
        self,
        *,
        file: core.File,
        output_format: typing.Optional[AllowedOutputFormats] = None,
        stem_variation_id: typing.Optional[MusicSeparateStemsRequestStemVariationId] = OMIT,
        sign_with_c_2_pa: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[bytes]:
        """
        Separate an audio file into individual stems. This endpoint might have high latency, depending on the length of the audio file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        output_format : typing.Optional[AllowedOutputFormats]
            Output format of the generated audio. Formatted as codec_sample_rate_bitrate. So an mp3 with 22.05kHz sample rate at 32kbs is represented as mp3_22050_32. MP3 with 192kbps bitrate requires you to be subscribed to Creator tier or above. PCM with 44.1kHz sample rate requires you to be subscribed to Pro tier or above. Note that the μ-law format (sometimes written mu-law, often approximated as u-law) is commonly used for Twilio audio inputs.

        stem_variation_id : typing.Optional[MusicSeparateStemsRequestStemVariationId]
            The id of the stem variation to use.

        sign_with_c_2_pa : typing.Optional[bool]
            Whether to sign the generated song with C2PA. Applicable only for mp3 files.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            ZIP archive containing separated audio stems. Each stem is provided as a separate audio file in the requested output format.
        """
        async with self._raw_client.separate_stems(
            file=file,
            output_format=output_format,
            stem_variation_id=stem_variation_id,
            sign_with_c_2_pa=sign_with_c_2_pa,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    @property
    def composition_plan(self):
        if self._composition_plan is None:
            from .composition_plan.client import AsyncCompositionPlanClient  # noqa: E402

            self._composition_plan = AsyncCompositionPlanClient(client_wrapper=self._client_wrapper)
        return self._composition_plan

    @property
    def finetunes(self):
        if self._finetunes is None:
            from .finetunes.client import AsyncFinetunesClient  # noqa: E402

            self._finetunes = AsyncFinetunesClient(client_wrapper=self._client_wrapper)
        return self._finetunes
